@@ -213,7 +213,7 @@
 
       type(ground_state_t), pointer :: gs
 
-      integer nns, noccs, i, ns_read, l
+      integer nns, noccs, i, ns_synth, ns_custom, l
       logical synthetic_atoms, found, reparametrize_pseudos
       real(dp) :: new_a, new_b, new_rmax
 
@@ -309,33 +309,11 @@ C Sanity checks on values
           basp%mass = atmass(abs(int(basp%z)))
         endif
 
-        if (.not.basp%bessel) then
-          write(6,"(/,a)") "---- Processing specs for species: " //
-     $        trim(basp%label)
-        endif
-
-        if (basp%bessel) then
-          ! Initialize a constant pseudo
-          call pseudo_init_constant(basp%pseudopotential)
-        else if (basp%synthetic) then
-          synthetic_atoms = .true.
-          ! Will set gs later
-          call pseudo_read(basp%ps_file_spec,basp%pseudopotential,
-     $         basp%psml_handle,basp%has_psml_ps,
-     $         new_grid=reparametrize_pseudos,a=new_a,b=new_b,
-     $         rmax=new_rmax,
-     $         debugging_enabled=write_ion_plot_files)
+        if (basp%synthetic) then
+           synthetic_atoms = .true.
         else
-          call ground_state(abs(int(basp%z)),basp%ground_state)
-          call pseudo_read(basp%ps_file_spec,basp%pseudopotential,
-     $         basp%psml_handle,basp%has_psml_ps,
-     $         new_grid=reparametrize_pseudos,a=new_a,b=new_b,
-     $         rmax=new_rmax,
-     $         debugging_enabled=write_ion_plot_files)
+           call ground_state(abs(int(basp%z)),basp%ground_state)
         endif
-!        if (reparametrize_pseudos.and. .not. basp%bessel)
-!     .    call pseudo_reparametrize(p=basp%pseudopotential,
-!     .                             a=new_a, b=new_b,label=basp%label)
 
       enddo
 
@@ -347,16 +325,24 @@ C Sanity checks on values
          if (synthetic_atoms)
      .          call die("Block SyntheticAtoms does not exist.")
       else
-        ns_read = 0
+        ns_synth = 0
+        ns_custom = 0
         do while(fdf_bline(bfdf, pline))
 
-          ns_read = ns_read + 1
           if (.not. fdf_bmatch(pline,'i'))
      .      call die("Wrong format in SyntheticAtoms")
           isp = fdf_bintegers(pline,1)
           if (isp .gt. nsp .or. isp .lt. 1)
      .      call die("Wrong specnum in SyntheticAtoms")
           basp => basis_parameters(isp)
+
+          if (.not. basp%synthetic) then
+             ! The information is for a normal species, whose
+             ! valence information needs to be changed
+             basp%custom_valence_config = .true.
+             basp%ground_state_tables = basp%ground_state
+          endif
+
           gs   => basp%ground_state
           if (.not. fdf_bline(bfdf, pline)) call die("No n info")
           nns = fdf_bnintegers(pline)
@@ -369,7 +355,7 @@ C Sanity checks on values
             gs%n(i-1) = fdf_bintegers(pline,i)
           enddo
           if (.not. fdf_bline(bfdf, pline))
-     .      call die("No occupation info in Synthetic block")
+     .      call die("No occupation info in SyntheticAtoms block")
           noccs = fdf_bnvalues(pline)
           if (noccs .lt. nns) call die("Need more occupations")
           gs%occupation(:) = 0.0_dp
@@ -386,28 +372,62 @@ C Sanity checks on values
           gs%occupied(0:4) = (gs%occupation .gt. 0.0_dp)
           gs%occupied(4) = .false.
           gs%z_valence = sum(gs%occupation(0:noccs-1))
-          write(6,'(a,i2)',advance='no')
-     .         'Ground state valence configuration (synthetic): '
+
+          if (basp%synthetic) then
+             write(6,'(a,i2)',advance='no')
+     .            trim(basp%label) //
+     .            'Ground state valence configuration (synthetic): '
+             ns_synth = ns_synth + 1
+          else
+             write(6,'(a,i2)',advance='no')
+     .            trim(basp%label) //
+     $            ': Ground state valence configuration (custom): '
+             ns_custom = ns_custom + 1
+          endif
           do l=0,3
-            if (gs%occupied(l))
-     .        write(6,'(2x,i1,a1,f8.5)',advance='no')
-     .          gs%n(l),sym(l), gs%occupation(l)
+             write(6,'(2x,i1,a1,f8.5)',advance='no')
+     .            gs%n(l),sym(l), gs%occupation(l)
           enddo
           write(6,'(a)') ''
 
         enddo
-        write(6,"(a,i2)") "Number of synthetic species: ", ns_read
+        if (ns_synth > 0) then
+           write(6,"(a,i2)") "Number of synthetic species: ", ns_synth
+        endif
+        if (ns_custom > 0) then
+           write(6,"(a,i2)")
+     $          "Number of species with custom GS valence config: ",
+     $          ns_custom
+        endif
       endif
-!
-!  Defer this here in case there are synthetic atoms
+
       do isp=1,nsp
         basp=>basis_parameters(isp)
+
+        if (.not.basp%bessel) then
+          write(6,"(/,a)") "---- Processing specs for species: " //
+     $        trim(basp%label)
+        endif
+
+        if (basp%bessel) then
+          ! Initialize a constant pseudo
+          call pseudo_init_constant(basp%pseudopotential)
+        else
+          call pseudo_read(basp%ps_file_spec,basp%pseudopotential,
+     $         basp%psml_handle,basp%has_psml_ps,
+     $         new_grid=reparametrize_pseudos,a=new_a,b=new_b,
+     $         rmax=new_rmax,
+     $         debugging_enabled=write_ion_plot_files)
+        endif
+
         if (basp%synthetic) then
           gs => basp%ground_state
           if (gs%z_valence .lt. 0.001)
      .      call die("Synthetic species not detailed")
         endif
+
         call semicore_check(isp)
+
       enddo
 
       call remass()
@@ -459,12 +479,11 @@ C Sanity checks on values
       call cnfig(z,gs%n(0:4))
 
       write(6,'(a,i2)',advance='no')
-     .     'Ground state valence configuration: '
+     .     'Ground state valence configuration (from tables): '
       gs%occupied(4) = .false.         !! always
       do l=0,3
         gs%occupied(l) =  (gs%occupation(l).gt.0.1d0)
-        if (gs%occupied(l))
-     .    write(6,'(2x,i1,a1,i2.2)',advance='no')
+        write(6,'(2x,i1,a1,i2.2)',advance='no')
      .       gs%n(l),sym(l),nint(gs%occupation(l))
       enddo
       write(6,'(a)') ''
@@ -723,8 +742,12 @@ C Sanity checks on values
           call get_n_semicore_shells(basp%pseudopotential,
      $                               basp%ground_state%n(0:3),
      $                               nsemic_shells)
+        else
+            nsemic_shells(:) = 0
         endif
 
+        basp%nsemic_shells(:) = nsemic_shells(:)
+
         !! Check whether there are optional type and ionic charge
         if (fdf_bnnames(pline) .eq. 2)
      .    basp%basis_type = fdf_bnames(pline,2)
@@ -924,21 +947,28 @@ C Sanity checks on values
         polarized_shell_ptr => null()
         allocate (basp%lshell(0:basp%lmxo))
 
-        loop_l: do l= 0, basp%lmxo
+        ! Scan the shells, allocate storage and decide on some heuristics
+
+        loop_l_scan: do l= 0, basp%lmxo
           ls => basp%lshell(l)
           call initialize(ls)
           ls%l = l
           will_have_polarization_orb = .false.
+          ! write(6,'(a,i0)') '--> building shell with l=', l
 
           ! Search for tmp_shells with given l
           nn = 0
           do ish= 1, basp%nshells_tmp
             s => basp%tmp_shell(ish)
-            if (s%l .eq. l) nn=nn+1
+            if (s%l .eq. l) then
+               nn=nn+1
+               ! write(6,'(a,i0)') '--> normal shell l=', s%l
+            endif
             if (s%polarized .and. (s%l == (l-1))) then
                ! Add a new shell, which might have nzeta=0
                nn = nn + 1
                will_have_polarization_orb = .true.
+               ! write(6,'(a,1x,i0)') '--> polarized shell l=', s%l
             endif
 
           enddo
@@ -976,13 +1006,33 @@ C Sanity checks on values
 
           endif
 
+          ! Save number of shells with given l and allocate space
           ls%nn = nn
-          if (nn.eq.0) then
+          if (nn > 0) then
+             allocate(ls%shell(1:nn))
+          else
+            if (basp%ground_state%occupied(l)) then
+               write(6,'(a,i1,a1)') 'ERROR: Missing shell ' //
+     $               'in PAO.Basis for ' //
+     $               trim(basp%label) // ': ',
+     $               basp%ground_state%n(l), sym(l)
+               call die("Missing occupied shell in PAO.Basis")
+            else
+               write(6,'(/a,i1,a1)') 'WARNING: Missing shell ' //
+     $               'in PAO.Basis for ' //
+     $               trim(basp%label) // ': ',
+     $               basp%ground_state%n(l), sym(l)
+               write(6,'(a/)') 'WARNING: Possible loss ' //
+     $               'of variational freedom'
+            endif
             !! What else do we do here?
-            cycle loop_l
           endif
-                                     !!
-          allocate(ls%shell(1:nn))
+
+       enddo loop_l_scan
+
+       ! Final building of data structure
+       loop_l: do l= 0, basp%lmxo
+          ls => basp%lshell(l)
           ! Collect previously allocated shells
           ind = 0
           do ish=1, basp%nshells_tmp
@@ -994,6 +1044,7 @@ C Sanity checks on values
               if (basp%non_perturbative_polorbs) then
                  if (s%polarized) then
                     ! Save for later. There should be just one 'polarized' shell
+                    ! write(6,'(a,1x,i0)') '--> marked as polarized:', s%l
                     polarized_shell_ptr => ls%shell(ind)
                     ! Remove markers (in new objects)
                     ls%shell(ind)%polarized = .false.
@@ -1057,9 +1108,9 @@ C Sanity checks on values
                endif
             endif  ! polarized shell for l-1
 
-          enddo
+          enddo ! over tmp shells
 
-          do i = 1, nn
+          do i = 1, ls%nn
              if (ls%shell(i)%n.eq.-1) then
                  print "(a,i1,a1,i1,a,i1,a)",
      $                "ERROR: " // trim(basp%label)//
@@ -1072,7 +1123,7 @@ C Sanity checks on values
           ! We have to sort by n value to avoid cases in which
           ! a perturbative polarization orbital comes first
           ! in the 'nsm' series
-          call sort_by_n(ls%shell)
+          if ( ls%nn > 0 ) call sort_by_n(ls%shell)
 
         enddo loop_l
         ! Destroy temporary shells in basp
@@ -1588,6 +1639,8 @@ c (according to atmass subroutine).
             if (nsemic_shells(l) > 0) basp%lmxo = max(l,basp%lmxo)
          enddo
 
+         basp%nsemic_shells(:) = nsemic_shells(:)
+
          ! Check whether we need to consider larger l's due to
          ! polarization orbitals (this is a new behavior -- in the
          ! case of perturbative pol orbs, the highest-l shell will remain empty)
